// TText.java

import java.awt.*;
import java.awt.event.*;
import java.awt.font.*;
import java.awt.geom.*;
import java.awt.image.*;

class THello extends Frame {
    public THello() {
        super("THello");
        add(new TextCanvas());
        addWindowListener(new WindowEventHandler());
        pack();
        show();
    }

    // Code to handle the closing of frame
    class WindowEventHandler extends WindowAdapter {
        public void windowClosing(WindowEvent e) {
            System.exit(0);
        }
    }

    // The main method...
    public static void main(String arg[]) {
        new THello();
    }
}

// Canvas to paint the "Hello! Graphics World!" text
class TextCanvas extends Canvas {
    Font font;
    FontMetrics fontMetrics;

    TextCanvas() {
        font = new Font("Dialog", Font.BOLD, 40);
        fontMetrics = getFontMetrics(font);
        setSize(400, 150);
    }

    public void paint(Graphics g) {
        // Set up the 2D graphics context
        Graphics2D g2D = (Graphics2D) g;

        // Say Hello!
        sayHello(g2D, 30, 50);

        // Say Graphics
        sayGraphics(g2D, 30, 100);

        // This code controls the rendering quality of the word
        // World. You can see the smoothening of the inclined lines.
        RenderingHints qualityHints = new RenderingHints(
                                          RenderingHints.KEY_ANTIALIASING,
                                          RenderingHints.VALUE_ANTIALIAS_ON);
        qualityHints.put(RenderingHints.KEY_RENDERING,
                         RenderingHints.VALUE_RENDER_QUALITY);
        g2D.setRenderingHints(qualityHints);
          
        // Say World!
        sayWorld(g2D,
                 fontMetrics.stringWidth("Graphics")+45,
                 100);
    }

    // This method draws the text "Hello!" with gradient-paint
    public void sayHello(Graphics2D g2D, int x, int y) {
        // Assign font type
        g2D.setFont(font);
            
        // Create a gradient paint object and assign it
        // to the 2D graphics context
        GradientPaint gp = new GradientPaint(
                                30.0f, 50.0f,
                                Color.blue,
                                fontMetrics.stringWidth("Hello!"),
                                fontMetrics.getHeight(),
                                Color.red);             
        g2D.setPaint(gp);

        // Draw the string Hello!
        g2D.drawString("Hello!", x, y);                                                 
    }

    // This method draws the text "Graphics" with texture-paint type
    // fill pattern
    public void sayGraphics(Graphics2D g2D, int x, int y) {
        // Create a texture paint
        BufferedImage bi = new BufferedImage(5, 5,
                                BufferedImage.TYPE_INT_RGB);
        Graphics2D big = bi.createGraphics();
        big.setColor(Color.magenta);
        big.fillRect(0,0,5,5);
        big.setColor(Color.black);
        big.fillOval(0,0,5,5);
        Rectangle r  = new Rectangle(0, 0, 5, 5);
        TexturePaint tp = new TexturePaint(bi, r);
        g2D.setPaint(tp);
        g2D.drawString("Graphics", 30, 100);
    }

    // This method draws the outline of the text "World!".
    public void sayWorld(Graphics2D g2D, int x, int y) {
        AffineTransform at = new AffineTransform();
        at.setToTranslation(x, y);
        at.shear(-0.5, 0.0);
        FontRenderContext frc = new FontRenderContext(at, false, false);

        // Text representation to say World!...
        TextLayout tl = new TextLayout("World!", font, frc);

        // Obtain the outline of the text in object tl, and draw
        // it by invoking the draw() method in graphics context.
        Shape outline = tl.getOutline(null);

        // Set the outline color to blue
        g2D.setColor(Color.blue);

        BasicStroke wideStroke = new BasicStroke(2.0f);
        g2D.setStroke(wideStroke);

        // Draw the outline
        g2D.draw(outline);
    }
}

