// TTexturePaint.java

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.image.*;

class TTexturePaint extends JFrame {
    DisplayCanvas canvas;
    
    public TTexturePaint() {
        // 1. Assign a title to the frame and get a handle on
        // the frame's content pane
        super("TTexturePaint");
        Container container = getContentPane();

        // 2. Add the canvas with rectangles
        canvas = new DisplayCanvas();
        container.add(canvas);

        // 3. Create a panel to display various textures (canvases)
        JPanel panel = new JPanel();
        panel.setLayout(new GridLayout(1,10));
        panel.setBorder(new TitledBorder(
            "Select a Texture by Clicking the Mouse..."));

        // 4. Create canvas objects displaying thumb nails of textures,
        // and add them to the panel.
        for (int i=0; i<10; i++) {
           panel.add(new SelectableCanvas(
                         "textures/texture"+(i+1)+".jpg", canvas));
        }

        // 5. Add the panel to the frame
        container.add(BorderLayout.SOUTH, panel);

        // 6. Add a frame closing listener and display the frame
        addWindowListener(new WindowEventHandler());
        pack(); // Packs around the contents with suitable size
        show(); // Display the frame
    }

    // 7. Code to handle closing of the frame
    class WindowEventHandler extends WindowAdapter {
        public void windowClosing(WindowEvent e) {
            System.exit(0);
        }
    }

    // 8. The main method...
    public static void main(String arg[]) {
        new TTexturePaint();
    }
}

// 9. Canvas to display the selected texture
class DisplayCanvas extends Canvas {
    Image displayImage;

    // Constructor
    DisplayCanvas() {
        // Add the mouse listener to receive a rectangle selection
        setBackground(Color.white);  // For canvas background color
        setSize(400, 225);  // Canvas width=400 height=225
    }

    public void setImage(Image image) {
        displayImage = image; 
    }
        
    public void paint(Graphics g) {
        // Create the graphics context object
        Graphics2D g2D = (Graphics2D) g;

        // Create a texture paint object and display it in a rectangle
        // by filling it.
        if (displayImage != null) {
            BufferedImage bi = new BufferedImage(
                               displayImage.getWidth(this),
                               displayImage.getHeight(this),
                               BufferedImage.TYPE_INT_RGB);
            bi.createGraphics().drawImage(displayImage, 0, 0, this);

            Rectangle2D rectangle = new Rectangle2D.Float(0, 0,
                                       displayImage.getWidth(this),
                                       displayImage.getHeight(this));

            TexturePaint tp = new TexturePaint(bi, rectangle);
            g2D.setPaint(tp);
            g2D.fill(new Rectangle2D.Float(0, 0, getWidth(), getHeight()));
        }
    }
}

// 10. Class to define thumb nail objects of textures. The
// user can select a texture by clicking the mouse on the
// corresponding thumb nail.
class SelectableCanvas extends Canvas {
    Image image;
    DisplayCanvas canvas;

    SelectableCanvas(String imageFile, DisplayCanvas canvas) {
        this.image = getToolkit().getImage(imageFile);
        this.canvas = canvas;

        addMouseListener(new MouseEventHandler());
        setBackground(Color.white);
        setSize(35, 35); // width and height of a thumb nail
    }
                                           
    public void paint(Graphics g) {
        g.drawImage(image, 0, 0, this);
    }

    // 11. Class to handle mouse events that are generated
    // when the mouse is clicked on the canvas.
    class MouseEventHandler extends MouseAdapter {
        public void mouseClicked(MouseEvent evt) {
            canvas.setImage(image);
            canvas.repaint();
        }
    }
}

