// TBufferedImage1.java

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;

class TBufferedImage1 extends JFrame {
    DisplayCanvas canvas;
    JRadioButton buffButton, nonBuffButton;
    JButton displayButton, clearButton;
    
    public TBufferedImage1() {
        // 1. Assign a title to the frame and get a handle on
        // the frame's content pane
        super("TBufferedImage1");
        Container container = getContentPane();

        // 2. Define and add a canvas that displays textures.
        canvas = new DisplayCanvas();
        container.add(canvas);

        // 3. Create a panel to add the control buttons. Add the
        // panel to the main frame.
        JPanel panel = new JPanel();
        panel.setLayout(new GridLayout(2, 2));
        panel.setBorder(new TitledBorder(
            "Select an Option and Display Image..."));

        buffButton = new JRadioButton("Buffered");
        buffButton.addActionListener(new ButtonListener());
        nonBuffButton = new JRadioButton("Non-Buffered", true);
        nonBuffButton.addActionListener(new ButtonListener());
        ButtonGroup group = new ButtonGroup();
        group.add(buffButton);
        group.add(nonBuffButton);
       
        displayButton = new JButton("Display");
        displayButton.addActionListener(new ButtonListener());
        clearButton = new JButton("Clear");
        clearButton.addActionListener(new ButtonListener());

        panel.add(nonBuffButton);
        panel.add(buffButton);
        panel.add(displayButton);
        panel.add(clearButton);

        container.add(BorderLayout.SOUTH, panel);

        // 4. Add a frame closing listener and display the frame
        addWindowListener(new WindowEventHandler());
        pack(); // Packs around the contents with suitable size
        show(); // Display the frame
    }

    // 5. Code to handle closing of the frame
    class WindowEventHandler extends WindowAdapter {
        public void windowClosing(WindowEvent e) {
            System.exit(0);
        }
    }

    // 6. The main method...
    public static void main(String arg[]) {
        new TBufferedImage1();
    }

    // 7. Action listener to handle the event fired by radio
    // buttons and push type buttons.
    class ButtonListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            Object obj =  e.getSource();

            if (obj instanceof JRadioButton) {
                JRadioButton button = (JRadioButton) obj;
                if (button.equals(buffButton)) {
                    canvas.buffered = true;
                }
                else if (button.equals(nonBuffButton)) {
                    canvas.buffered = false;
                }
            }

            if (obj instanceof JButton) {
                JButton button = (JButton) obj;
                if (button.equals(displayButton)) {
                    canvas.display = true;
                    canvas.repaint();
                }
                else if (button.equals(clearButton)) {
                    canvas.clear = true;
                    canvas.repaint();
                }
            }
        }
    } 
}

// 8. Canvas to display the selected texture
class DisplayCanvas extends Canvas {
    boolean display = false;
    boolean clear = false;
    boolean buffered = false;

    Image displayImage; // for texture image

    // Constructor
    DisplayCanvas() {
        // Create an image object using the specified file
        displayImage = Toolkit.getDefaultToolkit().getImage("texture.jpg");

        MediaTracker mt = new MediaTracker(this);
        mt.addImage(displayImage, 1);
        try {
            mt.waitForAll();
        } catch (Exception e) {
            System.out.println("Exception while loading.");
        }

        // If the image has an unknown width, the image is not created
        // by using the texture file. Therefore exit the program.  
        if (displayImage.getWidth(this) == -1) {
            System.out.println("*** Make sure you have the texture image "
                            + "(*.jpg) file in the same directory.***");
            System.exit(0);
        }

        setBackground(Color.white);  // For canvas background color
        setSize(400, 225);  // Canvas width=400 height=225
    }
        
    public void paint(Graphics g) {
        // Create the graphics context object
        Graphics2D g2D = (Graphics2D) g;

        if (display) {
            if (buffered) {
                // Create a buffered image of the size of canvas.
                BufferedImage bi = (BufferedImage) createImage(
                                        getWidth(),
                                        getHeight());  

                // Draw the texture image into the memory buffer.
                for (int i=0; i<getWidth(); i=i+displayImage.getWidth(this)) {
                    for (int j=0; j<getHeight(); j=j+displayImage.getHeight(this)) {
                        bi.createGraphics().drawImage(displayImage, i, j, this);
                    }
                }
                // Draw the buffered Image on to the screen
                g2D.drawImage(bi, 0,0, this);
            }
            // This block of code draws the texture directly onto the screen.
            else if (!buffered) {
                for (int i=0; i<getWidth(); i=i+displayImage.getWidth(this)) {
                    for (int j=0; j<getHeight(); j=j+displayImage.getHeight(this)) {
                       g2D.drawImage(displayImage, i, j, this);
                    }
                }    
            }
            display = false;
        }
        // Clears the canvas.
        else if (clear) {
            // Clear Canvas
            g2D.setColor(Color.white);
            g2D.clearRect(0, 0, getWidth(), getHeight());

            clear = false;
        }
    }
}
