// TRaster.java

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;

class TRaster extends JFrame {
    DisplayPanel displayPanel;
    JToggleButton flipButton;
    
    public TRaster() {
        // 1. Assign a title to the frame and get a handle on
        // the frame's content pane
        super("TRaster");
        Container container = getContentPane();

        // 2. Create and add the display panel
        displayPanel = new DisplayPanel();
        container.add(displayPanel);

        // 3. Create and add the button used to flip the image
        Box box = Box.createHorizontalBox();
        flipButton = new JToggleButton("Flip the Image");
        flipButton.addActionListener(new ButtonListener());
        // Add the button at the center of box using glue components
        box.add(Box.createHorizontalGlue()); 
        box.add(flipButton);
        box.add(Box.createHorizontalGlue());
        // Add the flip button to the bottom portion of frame
        container.add(box, BorderLayout.SOUTH);

        //  4. Add a frame closing listener and display the frame
        addWindowListener(new WindowEventHandler());
        setSize(450, 400); // Width and height
        show(); // Display the frame
    }

    // 5. Code to handle closing of the frame
    class WindowEventHandler extends WindowAdapter {
        public void windowClosing(WindowEvent e) {
            System.exit(0);
        }
    }

    // 6. The main method...
    public static void main(String arg[]) {
        new TRaster();
    }

    // 7. Button listener class
    class ButtonListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            // If the image is not flipped
            if (!displayPanel.flipped) {
               // Flip the image
               displayPanel.flipBufferedImage();
               displayPanel.bi = displayPanel.bi2;
               displayPanel.flipped = true;
            }
            // If the image has already been flipped
            else {
               // Prepare to display the normal image
               displayPanel.bi = displayPanel.bi1;
               displayPanel.flipped = false;
            }
            // Update the display panel
            displayPanel.repaint();
        }
    }
}

// 8. Definition of the display panel class
class DisplayPanel extends JPanel {
    BufferedImage bi, bi1, bi2;
    boolean flipped = false;

    // 9. Constructor
    DisplayPanel() {
        setBackground(Color.white);  // For canvas background color
        setSize(450, 400);  // panel width & height

        // Get the image file
        Image image = getToolkit().getImage("car.jpg");

        // Track the image loading
        MediaTracker mt = new MediaTracker(this);
        mt.addImage(image, 1);
        try {
            mt.waitForAll();
        } catch (Exception e) {
            System.out.println("Exception while loading image.");
        }

        // If the image has an unknown width, the image is not created
        // by using the specified file. Therefore exit the program!  
        if (image.getWidth(this) == -1) {
            System.out.println("*** Make sure you have the image "
                + "(car.jpg) file in the same directory.***");
            System.exit(0);
        }

        // Create the buffered image
        // NOTE: You need to use the image type ARGB to consider the
        //       effect of alpha value of an image.
        bi1 = new BufferedImage(image.getWidth(this),
                                image.getHeight(this),
                                BufferedImage.TYPE_INT_ARGB);
        Graphics2D big = bi1.createGraphics();
        // Draw the image into the buffer
        big.drawImage(image, 0, 0, this);
        bi = bi1; // Assign bi1 to bi for painting
    }

    // 10. This method performs flipping of the buffered image
    // by accessing the data buffers
    public void flipBufferedImage() {
        // Create a buffered image to store the flipped image.
        bi2 = new BufferedImage(bi1.getWidth(), bi1.getHeight(),
                                bi1.getType());

        // Retrieve the data buffers from the rasters of buffered images
        DataBuffer db1 = bi1.getRaster().getDataBuffer();
        DataBuffer db2 = bi2.getRaster().getDataBuffer();

        // Perform flipping by retrieving the pixels from db1 in the
        // reverse order and storing them in db2.
        for (int i=db1.getSize()-1, j=0; i>=0; --i, j++) {
            db2.setElem(j, db1.getElem(i));
        }
    }

    // 11. The update method...
    public void update(Graphics g) {
        g.clearRect(0, 0, getWidth(), getHeight());
        paintComponent(g);
    }

    // 12. The paint method...
    public void paintComponent(Graphics g) {
        super.paintComponent(g);

        // Create the graphics context object
        Graphics2D g2D = (Graphics2D) g;

        // Display the buffered image bi. bi refers to
        // either bi1 or bi2 depending on the context.
        g2D.drawImage(bi, 0, 0, this);
    }
}
